@@ -5,18 +5,15 @@ from .exceptions import MissingEndpointsError
 from .exceptions import RequestValidationError
 from .exceptions import ResponseValidationError
 from .wrappers import PyramidOpenAPIRequest
-from openapi_core import Spec
-from openapi_core.deserializing.media_types.factories import (
-    MediaTypeDeserializersFactory,
-)
-from openapi_core.unmarshalling.schemas.enums import UnmarshalContext
-from openapi_core.unmarshalling.schemas.factories import SchemaUnmarshallersFactory
-from openapi_core.validation.exceptions import InvalidSecurity
-from openapi_core.validation.request.validators import RequestValidator
-from openapi_core.validation.response.validators import ResponseValidator
-from openapi_schema_validator import OAS30Validator
-from openapi_spec_validator import validate_spec
+from jsonschema_path import SchemaPath
+from openapi_core.unmarshalling.request import V30RequestUnmarshaller
+from openapi_core.unmarshalling.request import V31RequestUnmarshaller
+from openapi_core.unmarshalling.response import V30ResponseUnmarshaller
+from openapi_core.unmarshalling.response import V31ResponseUnmarshaller
+from openapi_core.validation.request.exceptions import SecurityValidationError
+from openapi_spec_validator import validate
 from openapi_spec_validator.readers import read_from_filename
+from openapi_spec_validator.versions.shortcuts import get_spec_version
 from pathlib import Path
 from pyramid.config import Configurator
 from pyramid.config import PHASE0_CONFIG
@@ -36,6 +33,7 @@ from string import Template
 from urllib.parse import urlparse
 
 import hupper
+import json
 import logging
 import typing as t
 
@@ -48,6 +46,7 @@ def includeme(config: Configurator) -> None:
     config.add_view_deriver(openapi_view)
     config.add_directive("pyramid_openapi3_add_formatter", add_formatter)
     config.add_directive("pyramid_openapi3_add_deserializer", add_deserializer)
+    config.add_directive("pyramid_openapi3_add_unmarshaller", add_unmarshaller)
     config.add_directive("pyramid_openapi3_add_explorer", add_explorer_view)
     config.add_directive("pyramid_openapi3_spec", add_spec_view)
     config.add_directive("pyramid_openapi3_spec_directory", add_spec_view_directory)
@@ -86,9 +85,7 @@ def openapi_validated(request: Request) -> dict:
 
     if request.environ.get("pyramid_openapi3.validate_request"):
         openapi_request = PyramidOpenAPIRequest(request)
-        validated = settings["request_validator"].validate(
-            settings["spec"], openapi_request
-        )
+        validated = settings["request_validator"].unmarshal(openapi_request)
         return validated
 
     return {}  # pragma: no cover
@@ -111,7 +108,6 @@ def openapi_view(view: View, info: ViewDeriverInfo) -> View:
     if info.options.get("openapi"):
 
         def wrapper_view(context: Context, request: Request) -> Response:
-
             # We need this to be able to raise AttributeError if view code
             # accesses request.openapi_validated on a view that is marked
             # with openapi=False
@@ -151,7 +147,12 @@ def add_explorer_view(
     route: str = "/docs/",
     route_name: str = "pyramid_openapi3.explorer",
     template: str = "static/index.html",
-    ui_version: str = "4.15.5",
+    ui_version: str = "5.12.0",
+    ui_config: t.Optional[dict[str, t.Any]] = None,
+    oauth_config: t.Optional[dict[str, t.Any]] = None,
+    oauth_redirect_route: t.Optional[str] = None,
+    oauth_redirect_route_name: str = "pyramid_openapi3.explorer.oauth2-redirect",
+    oauth_redirect_html: str = "static/oauth2-redirect.html",
     permission: str = NO_PERMISSION_REQUIRED,
     apiname: str = "pyramid_openapi3",
 ) -> None:
@@ -161,11 +162,28 @@ def add_explorer_view(
     :param route_name: Route name that's being added
     :param template: Dotted path to the html template that renders Swagger UI response
     :param ui_version: Swagger UI version string
+    :param ui_config:
+        A dictionary conforming to the SwaggerUI API.
+        Any settings defined here will override those defined by default.
+    :param oauth_config:
+        If defined, then SwaggerUI.initOAuth will be invoked with the supplied config.
+    :param oauth_redirect_route:
+        URL path where the redirect will be served. By default the path is constructed
+        by appending a ``/oauth2-redirect`` path component to the ``route`` parameter.
+    :param oauth_redirect_route_name:
+        Route name for the redirect route.
+    :param oauth_redirect_html:
+        Dotted path to the html that renders the oauth2-redirect HTML.
     :param permission: Permission for the explorer view
     """
 
+    if oauth_redirect_route is None:
+        oauth_redirect_route = route.rstrip("/") + "/oauth2-redirect"
+
     def register() -> None:
-        resolved_template = AssetResolver().resolve(template)
+        asset_resolver = AssetResolver()
+        resolved_template = asset_resolver.resolve(template)
+        redirect_html = asset_resolver.resolve(oauth_redirect_html)
 
         def explorer_view(request: Request) -> Response:
             settings = config.registry.settings
@@ -176,9 +194,20 @@ def add_explorer_view(
                 )
             with open(resolved_template.abspath()) as f:
                 template = Template(f.read())
+                merged_ui_config = {
+                    "url": request.route_path(settings[apiname]["spec_route_name"]),
+                    "dom_id": "#swagger-ui",
+                    "deepLinking": True,
+                    "validatorUrl": None,
+                    "layout": "StandaloneLayout",
+                    "oauth2RedirectUrl": request.route_url(oauth_redirect_route_name),
+                }
+                if ui_config:
+                    merged_ui_config.update(ui_config)
                 html = template.safe_substitute(
                     ui_version=ui_version,
-                    spec_url=request.route_url(settings[apiname]["spec_route_name"]),
+                    ui_config=json.dumps(merged_ui_config),
+                    oauth_config=json.dumps(oauth_config),
                 )
             return Response(html)
 
@@ -187,6 +216,16 @@ def add_explorer_view(
             route_name=route_name, permission=permission, view=explorer_view
         )
 
+        def redirect_view(request: Request) -> FileResponse:
+            return FileResponse(redirect_html.abspath())
+
+        config.add_route(oauth_redirect_route_name, oauth_redirect_route)
+        config.add_view(
+            route_name=oauth_redirect_route_name,
+            permission=permission,
+            view=redirect_view,
+        )
+
     config.action((f"{apiname}_add_explorer",), register, order=PHASE0_CONFIG)
 
 
@@ -204,6 +243,13 @@ def add_deserializer(config: Configurator, name: str, func: t.Callable) -> None:
     reg[name] = func
 
 
+def add_unmarshaller(config: Configurator, name: str, func: t.Callable) -> None:
+    """Add support for configuring unmarshallers."""
+    config.registry.settings.setdefault("pyramid_openapi3_unmarshallers", {})
+    reg = config.registry.settings["pyramid_openapi3_unmarshallers"]
+    reg[name] = func
+
+
 def add_spec_view(
     config: Configurator,
     filepath: str,
@@ -232,8 +278,8 @@ def add_spec_view(
             hupper.get_reloader().watch_files([filepath])
         spec_dict, _ = read_from_filename(filepath)
 
-        validate_spec(spec_dict)
-        spec = Spec.create(spec_dict)
+        validate(spec_dict)
+        spec = SchemaPath.from_dict(spec_dict)
 
         def spec_view(request: Request) -> FileResponse:
             return FileResponse(filepath, request=request, content_type="text/yaml")
@@ -284,8 +330,8 @@ def add_spec_view_directory(
 
         spec_dict, _ = read_from_filename(str(path))
         spec_url = path.as_uri()
-        validate_spec(spec_dict, spec_url=spec_url)
-        spec = Spec.create(spec_dict, url=spec_url)
+        validate(spec_dict, base_uri=spec_url)
+        spec = SchemaPath.from_dict(spec_dict, base_uri=spec_url)
 
         config.add_static_view(route, str(path.parent), permission=permission)
         config.add_route(route_name, f"{route}/{path.name}")
@@ -301,38 +347,44 @@ def add_spec_view_directory(
 
 
 def _create_api_settings(
-    config: Configurator, filepath: str, route_name: str, spec: Spec
+    config: Configurator, filepath: str, route_name: str, spec: SchemaPath
 ) -> t.Dict:
     custom_formatters = config.registry.settings.get("pyramid_openapi3_formatters")
     custom_deserializers = config.registry.settings.get(
         "pyramid_openapi3_deserializers"
     )
+    custom_unmarshallers = config.registry.settings.get(
+        "pyramid_openapi3_unmarshallers"
+    )
 
-    media_type_deserializers_factory = MediaTypeDeserializersFactory(
-        custom_deserializers=custom_deserializers
-    )
-    schema_unmarshallers_request_factory = SchemaUnmarshallersFactory(
-        OAS30Validator,
-        custom_formatters=custom_formatters,
-        context=UnmarshalContext.REQUEST,
-    )
-    schema_unmarshallers_response_factory = SchemaUnmarshallersFactory(
-        OAS30Validator,
-        custom_formatters=custom_formatters,
-        context=UnmarshalContext.RESPONSE,
-    )
+    # switch unmarshaller based on spec version
+    spec_version = get_spec_version(spec.contents())
+    request_unmarshallers = {
+        "OpenAPIV3.0": V30RequestUnmarshaller,
+        "OpenAPIV3.1": V31RequestUnmarshaller,
+    }
+    response_unmarshallers = {
+        "OpenAPIV3.0": V30ResponseUnmarshaller,
+        "OpenAPIV3.1": V31ResponseUnmarshaller,
+    }
+    request_unmarshaller = request_unmarshallers[str(spec_version)]
+    response_unmarshaller = response_unmarshallers[str(spec_version)]
 
     return {
         "filepath": filepath,
         "spec_route_name": route_name,
         "spec": spec,
-        "request_validator": RequestValidator(
-            schema_unmarshallers_request_factory,
-            media_type_deserializers_factory=media_type_deserializers_factory,
+        "request_validator": request_unmarshaller(
+            spec,
+            extra_format_validators=custom_formatters,
+            extra_media_type_deserializers=custom_deserializers,
+            extra_format_unmarshallers=custom_unmarshallers,
         ),
-        "response_validator": ResponseValidator(
-            schema_unmarshallers_response_factory,
-            media_type_deserializers_factory=media_type_deserializers_factory,
+        "response_validator": response_unmarshaller(
+            spec,
+            extra_format_validators=custom_formatters,
+            extra_media_type_deserializers=custom_deserializers,
+            extra_format_unmarshallers=custom_unmarshallers,
         ),
     }
 
@@ -358,9 +410,9 @@ def register_routes(
                 root_factory = path_item.get(root_factory_ext)
                 config.add_route(
                     route_name,
-                    pattern=route_prefix + pattern
-                    if route_prefix is not None
-                    else pattern,
+                    pattern=(
+                        route_prefix + pattern if route_prefix is not None else pattern
+                    ),
                     factory=root_factory or None,
                 )
 
@@ -384,7 +436,7 @@ def openapi_validation_error(
     if isinstance(context, RequestValidationError):
         status_code = 400
         for error in context.errors:
-            if isinstance(error, InvalidSecurity):
+            if isinstance(error, SecurityValidationError):
                 status_code = 401
 
     if isinstance(context, ResponseValidationError):
@@ -448,7 +500,7 @@ def check_all_routes(event: ApplicationCreated) -> None:
                 settings["pyramid_openapi3"]["routes"][route_name] = name
 
 
-def _get_server_prefixes(spec: Spec) -> t.List[str]:
+def _get_server_prefixes(spec: SchemaPath) -> t.List[str]:
     """Build a set of possible route prefixes from the api spec.
 
     Api routes may optionally be prefixed using servers (e.g: `/api/v1`).
