@@ -1,17 +1,18 @@
 """Exceptions used in pyramid_openapi3."""
 
-from openapi_core.schema.exceptions import OpenAPIError
-from openapi_core.unmarshalling.schemas.exceptions import InvalidSchemaFormatValue
+from dataclasses import dataclass
+from openapi_core.exceptions import OpenAPIError
+from openapi_core.unmarshalling.schemas.exceptions import UnmarshallerError
 from pyramid.httpexceptions import HTTPBadRequest
 from pyramid.httpexceptions import HTTPInternalServerError
 from pyramid.request import Request
 from pyramid.response import Response
 
-import attr
 import typing as t
 
 
 class RequestValidationError(HTTPBadRequest):
+    """Error raised when Request validation fails."""
 
     explanation = "Request validation failed."
 
@@ -28,6 +29,7 @@ class RequestValidationError(HTTPBadRequest):
 
 
 class ResponseValidationError(HTTPInternalServerError):
+    """Error raised when Response validation fails."""
 
     explanation = "Response validation failed."
 
@@ -48,11 +50,14 @@ class ResponseValidationError(HTTPInternalServerError):
         return str(self.detail) if self.detail else self.explanation
 
 
-@attr.s(hash=True)
-class InvalidCustomFormatterValue(InvalidSchemaFormatValue):
+@dataclass
+class InvalidCustomFormatterValue(UnmarshallerError):
     """Value failed to format with a custom formatter."""
 
-    field: t.Any = attr.ib()
+    field: str
+    value: str
+    type: str  # noqa: A003 # we use `type` as a dataclass field name
+    original_exception: Exception
 
     def __str__(self) -> str:
         """Provide more control over error message."""
@@ -64,7 +69,7 @@ class ImproperAPISpecificationWarning(UserWarning):
 
 
 def extract_errors(
-    request: Request, errors: t.List[OpenAPIError]
+    request: Request, errors: t.List[OpenAPIError], parent_field: t.Optional[str] = None
 ) -> t.Iterator[t.Dict[str, str]]:
     """Extract errors for JSON response.
 
@@ -107,32 +112,32 @@ def extract_errors(
     ```
     """
     for err in errors:
-        if getattr(err, "schema_errors", None) is not None:
-            yield from extract_errors(request, err.schema_errors)
+        schema_errors = getattr(err.__cause__, "schema_errors", None)
+        if schema_errors is not None:
+            yield from extract_errors(
+                request, schema_errors, getattr(err, "name", None)
+            )
             continue
 
         output = {"exception": err.__class__.__name__}
 
-        if getattr(err, "message", None) is not None:
-            message = err.message
-        else:
+        message = getattr(err, "message", None)
+        if getattr(err, "__cause__", None):
+            message = str(err.__cause__)
+        if message is None:
             message = str(err)
 
         output.update({"message": message})
 
-        field = None
-        if getattr(err, "field", None) is not None:
-            field = err.field
-        elif getattr(err, "name", None) is not None:
-            field = err.name
-        elif (
-            getattr(err, "validator", None) is not None and err.validator == "required"
-        ):
-            field = "/".join(err.validator_value)
-        elif (
-            getattr(err, "path", None) and err.path[0] and isinstance(err.path[0], str)
-        ):
-            field = "/".join([str(part) for part in err.path])
+        field = getattr(err, "field", parent_field)
+        if field is None:
+            field = getattr(err, "name", None)
+        if field is None and getattr(err, "validator", None) == "required":
+            field = "/".join(getattr(err, "validator_value", []))
+        if field is None:
+            path = getattr(err, "path", None)
+            if path and path[0] and isinstance(path[0], str):
+                field = "/".join([str(part) for part in path])
 
         if field:
             output.update({"field": field})
@@ -141,9 +146,11 @@ def extract_errors(
 
 
 class MissingEndpointsError(Exception):
+    """Error raised when endpoints are not found."""
+
     missing: list
 
     def __init__(self, missing: t.List[str]) -> None:
         self.missing = missing
         message = f"Unable to find routes for endpoints: {', '.join(missing)}"
-        super(MissingEndpointsError, self).__init__(message)
+        super().__init__(message)
